# Copyright (c) 2007, David Bender, Benegon Enterprises LLC
# All rights reserved.

# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

#   * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
#   * Neither the name of the Benegon Enterprises LLC nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The diskegrator is hosted at
# http://diskegrator.sourceforge.net

import subprocess
import os
import shutil
import bootloader
import tempfile

shareDir = '/lib/grub/x86_64-pc'

class Grub(bootloader.Bootloader):
	def __init__(self, grubConf, index):
		# FIXME check for grub.conf
		# FIXME check for proper directory structure
		self.conf = grubConf
		self.index = index

	def install(self, rootDev):
		mountpt = tempfile.mkdtemp(prefix='/tmp/diskegrator.grub.')

		# Mount boot partition
		cmd = ["mount", rootDev + str(self.index), mountpt]
		os.mkdir(mountpt +"/grub")
		retval = subprocess.Popen(cmd).wait()
		if retval != 0:
			raise 'Unable to mount boot partition!'
		
		try:
			# Copy common grub files to /grub dir
			cmd = ['cp', '-ra', shareDir, mountpt +"/grub"]
			print cmd
			retval = subprocess.Popen(cmd).wait()
			if retval != 0:
				raise 'Unable to copy grub files!'

			# Copy grub.conf
			shutil.copy(self.conf, mountpt+'/grub/menu.lst')

		finally:
			cmd = ["umount", mountpt]
			retval = subprocess.Popen(cmd).wait()

		# Run grub installation
		cmd = ["grub", "--batch", "--no-floppy"]
		proc = subprocess.Popen(cmd, stdin=subprocess.PIPE)
		grubcmd = "device (hd0) " + rootDev + "\n"
		grubcmd += "root (hd0,"+str(self.index-1)+")\n"
		grubcmd += "setup (hd0)\n"
		proc.communicate(grubcmd)
		retval = proc.wait()
