# Copyright (c) 2007, David Bender, Benegon Enterprises LLC
# All rights reserved.

# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

#     * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 #    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
  #   * Neither the name of the Benegon Enterprises LLC nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# The diskegrator is hosted at
# http://diskegrator.sourceforge.net

import sys
import subprocess
import os
import tempfile

class Partition:
	def __init__(self, size, type):
		self.size = size
		self.type = type
		pass
	
	def length(self):
		return self.size
	
	def type(self):
		return self.type

	# Given a block device node, install data into the device node
	def populate(self, devNode):
		pass


class Swap(Partition):
	def __init__(self, size):
		Partition.__init__(self, size, 0x82)
		
	def populate(self, devNode):
		# Perform mkswap
		cmd = ['mkswap', devNode]
		retval = subprocess.Popen(cmd).wait()
		return (retval == 0)


class ImageFile(Partition):
	def __init__(self, filename):
		# Check for filename and determine size
		if not os.path.exists(filename):
			raise "ImageFile " + filename + " not found!"
		self.filename = filename
		size = os.stat(filename)[6]
		# FIXME: for now assume linux partition
		Partition.__init__(self, size, 0x83)

	def populate(self, devNode):
		cmd = ['dd', 'if='+self.filename, 'of='+devNode]
		retval = subprocess.Popen(cmd).wait()
		if retval != 0:
			raise 'Unable to write image ' +self.filename+ ' to ' +devNode


class FileSystem(Partition):
	def __init__(self, fstype, size, srcData=None, options=None):
		print "Partition", size
		# FIXME: check that fstype is supported
		# FIXME: for now assume linux partition
		# FIXME: raise error if specified size is not sufficient for srcData
		self.fstype = fstype
		self.srcData = srcData
		self.options = options
		Partition.__init__(self, size, 0x83)
		
	def populate(self, devNode):
		cmd = ['mkfs.'+self.fstype, devNode]
		# FIXME Hack to make reiserfs non-interactive
		# a better solution would be to pass command line arguments
		if self.fstype == 'reiserfs':
			cmd.append('-ff')
		print cmd
		retval = subprocess.Popen(cmd).wait()
		if retval != 0:
			raise 'Unable to create filesystem ' +self.fstype+ ' at ' +devNode

		if self.srcData:
			try:
				mountpt = tempfile.mkdtemp(prefix='/tmp/diskegrator.fs.')
				# Mount the filesystem
				cmd = ["mount", devNode, mountpt]
				retval = subprocess.Popen(cmd).wait()
				if retval != 0:
					raise 'Unable to mount ' + devNode + '!'

				# Copy source data
				cmd = ["tar", "-C", self.srcData, "-cf" "-", "."]
				cmd2 = ["tar", "-C", mountpt, "-xf", "-"]
				x = subprocess.Popen(cmd, stdout=subprocess.PIPE)
				retval = subprocess.Popen(cmd2, stdin=x.stdout).wait()
				if retval != 0:
					raise 'Unable to copy data from ' + self.srcData + ' to mnt!'

			finally:
				# Unmount the filesystem
				cmd = ["umount", mountpt]
				retval = subprocess.Popen(cmd).wait()
				if retval != 0:
					raise 'Unable to unmount ' + devNode + '!'
